% Monte Carlo Simulations
clear
rep = 1000;
option1 = optimoptions(@fminunc,'Algorithm','trust-region','GradObj','on','Hessian','on', 'Display','notify','DerivativeCheck','off');
option2 = optimoptions(@fminunc,'Algorithm','trust-region','GradObj','on','Hessian','off','Display','notify','DerivativeCheck','off');

% random number generator seed
ssd = 20200508;
rng(ssd);

% true parameters
lam = 2;
gam = [-1,2];

% generating W
rook = 1; % rook = 1 if rook network and rook = 0 otherwise
if rook == 1 
    n  = 2500;
    W1 = zeros(n);
    for i = 1:(n-1)
        for j = (i+1):n
            if (j==i+1) || (j==i+51)
                W1(i,j) = 1;
            end
        end
    end
    W1 = W1+W1';
else
    n  = 2500;
    W1 = zeros(n);
    for i = 1:(n-1)
        for j = (i+1):n
            if (j==i+1) || (j==i+50) || (j==i+51) || (j==i+52)
                W1(i,j) = 1;
            end
        end
    end
    W1 = W1+W1';
end
Wn = wnorm(W1);
Wn = sparse(Wn);

K1 = 3;
delta1 = zeros(K1,rep);
delta2 = zeros(K1,rep);

ns = floor(sr*n); % sampled data
for s = 1:rep
disp(s);
J1 = speye(n);
Jn = J1(randperm(n,ns),:);

%Xn = randn(n,1);
Xn = [ones(n,1),2*rand(n,1)-1];
En = random('logistic',0,1,[n,1]);

b0 = [lam,gam]';

P0 = zeros(n,1);

u1 = exp([Wn*P0,Xn]*b0);
Pt = u1./(1+u1);

while max(abs(Pt-P0)) > 1e-008
    P0 = Pt;
    u1 = exp([Wn*P0,Xn]*b0);
    Pt = u1./(1+u1);
end
Yn = ([Wn*Pt,Xn]*b0>En);
JY = Jn*Yn;
%% correct model
% initial estimation
Pn = 0.5*ones(n,1);
WP = Wn*Pn;

Zn = Jn*[WP,Xn];
b1 = zeros(K1,1);
bhat = fminunc('logit_obj',b1,option1,JY,Zn);
% NPL estimation
while max(abs(bhat-b1)) > 1e-008
    %tic
    b1 = bhat;
    
    u1 = exp([WP,Xn]*b1);
    Pn = u1./(1+u1);
    WP = Wn*Pn;
    
    Zn = Jn*[WP,Xn];
    bhat = fminunc('logit_obj',b1,option1,JY,Zn);
    %toc
end
delta1(:,s) = bhat;
%% misspecified model
% initial estimation
Ws = Jn*W1*Jn';
Ws = wnorm(Ws);
Ws = sparse(Ws);

Ps = 0.5*ones(ns,1);
WP = Ws*Ps;

JX = Jn*Xn;

Zs = [WP,JX];
b1 = zeros(K1,1);
bhat = fminunc('logit_obj',b1,option1,JY,Zs);
% NPL estimation
while max(abs(bhat-b1)) > 1e-008
    %tic
    b1 = bhat;
    
    u1 = exp(Zs*b1);
    Ps = u1./(1+u1);
    WP = Ws*Ps;
    
    Zs = [WP,JX];
    bhat = fminunc('logit_obj',b1,option1,JY,Zs);
    %toc
end
delta2(:,s) = bhat;
end
%%
fid = fopen('binary_sar_mc_output.txt','a');
fprintf(fid,'%s \n',date);
fprintf(fid,'# of replications = %8.1f \n',rep);
fprintf(fid,'sample size = %8.1f \n',n);
fprintf(fid,'sampling rate = %8.2f \n',sr);
fprintf(fid,'*******************\n');
if rook == 1
    fprintf(fid,'rook network\n');
else
    fprintf(fid,'queen network\n');
end
fprintf(fid,'*******************\n');
fprintf(fid,'NPL-1\n');
p2mean = mean(delta2,2);
p2sdev = std(delta2,0,2);
for ip = 1:K1
    fprintf(fid,'%3.3f(%3.3f) \n',p2mean(ip),p2sdev(ip));
end
fprintf(fid,'*******************\n');
fprintf(fid,'NPL-2\n');
p1mean = mean(delta1,2);
p1sdev = std(delta1,0,2);
for ip = 1:K1
    fprintf(fid,'%3.3f(%3.3f) \n',p1mean(ip),p1sdev(ip));
end
fprintf(fid,'*******************\n\n');
fclose(fid);